@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,6 +28,19 @@ class CDiscoveryRule extends CItemGeneral {
 	protected $tableAlias = 'i';
 	protected $sortColumns = ['itemid', 'name', 'key_', 'delay', 'type', 'status'];
 
+	/**
+	 * Define a set of supported pre-processing rules.
+	 *
+	 * @var array
+	 *
+	 * 5.6 would allow this to be defined constant.
+	 */
+	public static $supported_preprocessing_types = [ZBX_PREPROC_REGSUB, ZBX_PREPROC_JSONPATH,
+		ZBX_PREPROC_VALIDATE_NOT_REGEX, ZBX_PREPROC_ERROR_FIELD_JSON, ZBX_PREPROC_THROTTLE_TIMED_VALUE,
+		ZBX_PREPROC_SCRIPT, ZBX_PREPROC_PROMETHEUS_TO_JSON, ZBX_PREPROC_XPATH, ZBX_PREPROC_ERROR_FIELD_XML,
+		ZBX_PREPROC_CSV_TO_JSON
+	];
+
 	public function __construct() {
 		parent::__construct();
 
@@ -54,38 +67,41 @@ class CDiscoveryRule extends CItemGeneral {
 		];
 
 		$defOptions = [
-			'groupids'					=> null,
-			'templateids'				=> null,
-			'hostids'					=> null,
-			'itemids'					=> null,
-			'interfaceids'				=> null,
-			'inherited'					=> null,
-			'templated'					=> null,
-			'monitored'					=> null,
-			'editable'					=> false,
-			'nopermissions'				=> null,
+			'groupids'						=> null,
+			'templateids'					=> null,
+			'hostids'						=> null,
+			'itemids'						=> null,
+			'interfaceids'					=> null,
+			'inherited'						=> null,
+			'templated'						=> null,
+			'monitored'						=> null,
+			'editable'						=> false,
+			'nopermissions'					=> null,
 			// filter
-			'filter'					=> null,
-			'search'					=> null,
-			'searchByAny'				=> null,
-			'startSearch'				=> false,
-			'excludeSearch'				=> false,
-			'searchWildcardsEnabled'	=> null,
+			'filter'						=> null,
+			'search'						=> null,
+			'searchByAny'					=> null,
+			'startSearch'					=> false,
+			'excludeSearch'					=> false,
+			'searchWildcardsEnabled'		=> null,
 			// output
-			'output'					=> API_OUTPUT_EXTEND,
-			'selectHosts'				=> null,
-			'selectItems'				=> null,
-			'selectTriggers'			=> null,
-			'selectGraphs'				=> null,
-			'selectHostPrototypes'		=> null,
-			'selectFilter'				=> null,
-			'countOutput'				=> false,
-			'groupCount'				=> false,
-			'preservekeys'				=> false,
-			'sortfield'					=> '',
-			'sortorder'					=> '',
-			'limit'						=> null,
-			'limitSelects'				=> null
+			'output'						=> API_OUTPUT_EXTEND,
+			'selectHosts'					=> null,
+			'selectItems'					=> null,
+			'selectTriggers'				=> null,
+			'selectGraphs'					=> null,
+			'selectHostPrototypes'			=> null,
+			'selectApplicationPrototypes'	=> null,
+			'selectFilter'					=> null,
+			'selectLLDMacroPaths'			=> null,
+			'selectPreprocessing'			=> null,
+			'countOutput'					=> false,
+			'groupCount'					=> false,
+			'preservekeys'					=> false,
+			'sortfield'						=> '',
+			'sortorder'						=> '',
+			'limit'							=> null,
+			'limitSelects'					=> null
 		];
 		$options = zbx_array_merge($defOptions, $options);
 
@@ -188,6 +204,12 @@ class CDiscoveryRule extends CItemGeneral {
 
 		// search
 		if (is_array($options['search'])) {
+			if (array_key_exists('error', $options['search']) && $options['search']['error'] !== null) {
+				zbx_db_search('item_rtdata ir', ['search' => ['error' => $options['search']['error']]] + $options,
+					$sqlParts
+				);
+			}
+
 			zbx_db_search('items i', $options, $sqlParts);
 		}
 
@@ -202,6 +224,12 @@ class CDiscoveryRule extends CItemGeneral {
 				$options['filter']['lifetime'] = getTimeUnitFilters($options['filter']['lifetime']);
 			}
 
+			if (array_key_exists('state', $options['filter']) && $options['filter']['state'] !== null) {
+				$this->dbFilter('item_rtdata ir', ['filter' => ['state' => $options['filter']['state']]] + $options,
+					$sqlParts
+				);
+			}
+
 			$this->dbFilter('items i', $options, $sqlParts);
 
 			if (isset($options['filter']['host'])) {
@@ -222,16 +250,16 @@ class CDiscoveryRule extends CItemGeneral {
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($item = DBfetch($res)) {
-			if ($options['countOutput']) {
-				if ($options['groupCount']) {
-					$result[] = $item;
-				}
-				else {
-					$result = $item['rowscount'];
-				}
+			if (!$options['countOutput']) {
+				$result[$item['itemid']] = $item;
+				continue;
+			}
+
+			if ($options['groupCount']) {
+				$result[] = $item;
 			}
 			else {
-				$result[$item['itemid']] = $item;
+				$result = $item['rowscount'];
 			}
 		}
 
@@ -323,10 +351,25 @@ class CDiscoveryRule extends CItemGeneral {
 			}
 
 			// Option 'Convert to JSON' is not supported for discovery rule.
-			unset($item['output_format']);
+			unset($item['itemid'], $item['output_format']);
 		}
 		unset($item);
 
+		// Get only hosts not templates from items
+		$hosts = API::Host()->get([
+			'output' => [],
+			'hostids' => zbx_objectValues($items, 'hostid'),
+			'preservekeys' => true
+		]);
+		foreach ($items as &$item) {
+			if (array_key_exists($item['hostid'], $hosts)) {
+				$item['rtdata'] = true;
+			}
+		}
+		unset($item);
+
+		$this->validateCreateLLDMacroPaths($items);
+		$this->validateDependentItems($items);
 		$this->createReal($items);
 		$this->inherit($items);
 
@@ -344,15 +387,19 @@ class CDiscoveryRule extends CItemGeneral {
 		$items = zbx_toArray($items);
 
 		$db_items = $this->get([
-			'output' => ['itemid', 'name', 'type', 'authtype', 'allow_traps', 'retrieve_mode'],
+			'output' => ['itemid', 'name', 'type', 'master_itemid', 'authtype', 'allow_traps', 'retrieve_mode'],
 			'selectFilter' => ['evaltype', 'formula', 'conditions'],
 			'itemids' => zbx_objectValues($items, 'itemid'),
 			'preservekeys' => true
 		]);
 
 		$this->checkInput($items, true, $db_items);
+		$this->validateUpdateLLDMacroPaths($items);
 
-		$items = $this->extendFromObjects(zbx_toHash($items, 'itemid'), $db_items, ['flags', 'type']);
+		$items = $this->extendFromObjects(zbx_toHash($items, 'itemid'), $db_items, ['flags', 'type', 'authtype',
+			'master_itemid'
+		]);
+		$this->validateDependentItems($items);
 
 		$defaults = DB::getDefaults('items');
 		$clean = [
@@ -407,10 +454,9 @@ class CDiscoveryRule extends CItemGeneral {
 				}
 			}
 
-			if ($db_items[$item['itemid']]['type'] == ITEM_TYPE_HTTPAGENT) {
-				// Clean username and password on authtype change to HTTPTEST_AUTH_NONE.
-				if (array_key_exists('authtype', $item) && $item['authtype'] == HTTPTEST_AUTH_NONE
-						&& $item['authtype'] != $db_items[$item['itemid']]['authtype']) {
+			if ($item['type'] == ITEM_TYPE_HTTPAGENT) {
+				// Clean username and password when authtype is set to HTTPTEST_AUTH_NONE.
+				if ($item['authtype'] == HTTPTEST_AUTH_NONE) {
 					$item['username'] = '';
 					$item['password'] = '';
 				}
@@ -591,8 +637,8 @@ class CDiscoveryRule extends CItemGeneral {
 	 * the user doesn't have the necessary permissions.
 	 *
 	 * @param array $data
-	 * @param array $data['discoveryruleids']	An array of item ids to be cloned
-	 * @param array $data['hostids']			An array of host ids were the items should be cloned to
+	 * @param array $data['discoveryids']  An array of item ids to be cloned.
+	 * @param array $data['hostids']       An array of host ids were the items should be cloned to.
 	 *
 	 * @return bool
 	 */
@@ -642,6 +688,8 @@ class CDiscoveryRule extends CItemGeneral {
 			'output' => $output,
 			'hostids' => $data['templateids'],
 			'selectFilter' => ['formula', 'evaltype', 'conditions'],
+			'selectLLDMacroPaths' => ['lld_macro', 'path'],
+			'selectPreprocessing' => ['type', 'params', 'error_handler', 'error_handler_params'],
 			'preservekeys' => true
 		]);
 		$json = new CJson();
@@ -686,7 +734,8 @@ class CDiscoveryRule extends CItemGeneral {
 		$srcTriggers = API::TriggerPrototype()->get([
 			'discoveryids' => $srcDiscovery['itemid'],
 			'output' => ['triggerid', 'expression', 'description', 'url', 'status', 'priority', 'comments',
-				'templateid', 'type', 'recovery_mode', 'recovery_expression', 'correlation_mode', 'correlation_tag'
+				'templateid', 'type', 'recovery_mode', 'recovery_expression', 'correlation_mode', 'correlation_tag',
+				'opdata'
 			],
 			'selectHosts' => API_OUTPUT_EXTEND,
 			'selectItems' => ['itemid', 'type'],
@@ -866,22 +915,36 @@ class CDiscoveryRule extends CItemGeneral {
 		return $result;
 	}
 
-	protected function createReal(&$items) {
+	protected function createReal(array &$items) {
+		$items_rtdata = [];
+		$create_items = [];
+
 		// create items without formulas, they will be updated when items and conditions are saved
-		$createItems = [];
-		foreach ($items as $item) {
-			if (isset($item['filter'])) {
+		foreach ($items as $key => $item) {
+			if (array_key_exists('filter', $item)) {
 				$item['evaltype'] = $item['filter']['evaltype'];
 				unset($item['filter']);
 			}
 
-			$createItems[] = $item;
+			if (array_key_exists('rtdata', $item)) {
+				$items_rtdata[$key] = [];
+				unset($item['rtdata']);
+			}
+
+			$create_items[] = $item;
 		}
-		$createItems = DB::save('items', $createItems);
+		$create_items = DB::save('items', $create_items);
+
+		foreach ($items_rtdata as $key => &$value) {
+			$value['itemid'] = $create_items[$key]['itemid'];
+		}
+		unset($value);
+
+		DB::insert('item_rtdata', $items_rtdata, false);
 
 		$conditions = [];
 		foreach ($items as $key => &$item) {
-			$item['itemid'] = $createItems[$key]['itemid'];
+			$item['itemid'] = $create_items[$key]['itemid'];
 
 			// conditions
 			if (isset($item['filter'])) {
@@ -896,20 +959,35 @@ class CDiscoveryRule extends CItemGeneral {
 
 		$conditions = DB::save('item_condition', $conditions);
 
-		// update formulas
-		$itemConditions = [];
+		$item_conditions = [];
+
 		foreach ($conditions as $condition) {
-			$itemConditions[$condition['itemid']][] = $condition;
+			$item_conditions[$condition['itemid']][] = $condition;
 		}
+
+		$lld_macro_paths = [];
+
 		foreach ($items as $item) {
+			// update formulas
 			if (isset($item['filter']) && $item['filter']['evaltype'] == CONDITION_EVAL_TYPE_EXPRESSION) {
-				$this->updateFormula($item['itemid'], $item['filter']['formula'], $itemConditions[$item['itemid']]);
+				$this->updateFormula($item['itemid'], $item['filter']['formula'], $item_conditions[$item['itemid']]);
+			}
+
+			// $item['lld_macro_paths'] expects to be filled with validated fields 'lld_macro' and 'path' and values.
+			if (array_key_exists('lld_macro_paths', $item)) {
+				foreach ($item['lld_macro_paths'] as $lld_macro_path) {
+					$lld_macro_paths[] = $lld_macro_path + ['itemid' => $item['itemid']];
+				}
 			}
 		}
+
+		DB::insertBatch('lld_macro_path', $lld_macro_paths);
+
+		$this->createItemPreprocessing($items);
 	}
 
-	protected function updateReal($items) {
-		$items = zbx_toArray($items);
+	protected function updateReal(array $items) {
+		CArrayHelper::sort($items, ['itemid']);
 
 		$ruleIds = zbx_objectValues($items, 'itemid');
 
@@ -970,12 +1048,144 @@ class CDiscoveryRule extends CItemGeneral {
 			}
 		}
 
-		// update formulas
+		$itemids = [];
+		$lld_macro_paths = [];
+		$db_lld_macro_paths = [];
+
 		foreach ($items as $item) {
+			// update formulas
 			if (isset($item['filter']) && $item['filter']['evaltype'] == CONDITION_EVAL_TYPE_EXPRESSION) {
 				$this->updateFormula($item['itemid'], $item['filter']['formula'], $ruleConditions[$item['itemid']]);
 			}
+
+			// "lld_macro_paths" could be empty or filled with fields "lld_macro", "path" or "lld_macro_pathid".
+			if (array_key_exists('lld_macro_paths', $item)) {
+				$itemids[$item['itemid']] = true;
+
+				if ($item['lld_macro_paths']) {
+					foreach ($item['lld_macro_paths'] as $lld_macro_path) {
+						$lld_macro_paths[] = $lld_macro_path + ['itemid' => $item['itemid']];
+					}
+				}
+			}
 		}
+
+		// Gather all existing LLD macros from given discovery rules.
+		if ($itemids) {
+			$db_lld_macro_paths = DB::select('lld_macro_path', [
+				'output' => ['lld_macro_pathid', 'itemid', 'lld_macro', 'path'],
+				'filter' => ['itemid' => array_keys($itemids)]
+			]);
+		}
+
+		/*
+		 * DB::replaceByPosition() does not allow to change records one by one due to unique indexes on two table
+		 * columns. Problems arise when given records are the same as records in DB and they are sorted differently.
+		 * That's why checking differences between old and new records is done manually.
+		 */
+
+		$lld_macro_paths_to_update = [];
+
+		foreach ($lld_macro_paths as $idx1 => $lld_macro_path) {
+			foreach ($db_lld_macro_paths as $idx2 => $db_lld_macro_path) {
+				if (array_key_exists('lld_macro_pathid', $lld_macro_path)) {
+					// Update records by primary key.
+
+					// Find matching "lld_macro_pathid" and update fields accordingly.
+					if (bccomp($lld_macro_path['lld_macro_pathid'], $db_lld_macro_path['lld_macro_pathid']) == 0) {
+						$fields_to_update = [];
+
+						if (array_key_exists('lld_macro', $lld_macro_path)
+								&& $lld_macro_path['lld_macro'] === $db_lld_macro_path['lld_macro']) {
+							// If same "lld_macro" is found in DB, update only "path" if necessary.
+
+							if (array_key_exists('path', $lld_macro_path)
+									&& $lld_macro_path['path'] !== $lld_macro_path['path']) {
+								$fields_to_update['path'] = $lld_macro_path['path'];
+							}
+						}
+						else {
+							/*
+							 * Update all other fields that correspond to given "lld_macro_pathid". Except for primary
+							 * key "lld_macro_pathid" and "itemid".
+							 */
+
+							foreach ($lld_macro_path as $field => $value) {
+								if ($field !== 'itemid' && $field !== 'lld_macro_pathid') {
+									$fields_to_update[$field] = $value;
+								}
+							}
+						}
+
+						/*
+						 * If there are any changes made, update fields in DB. Otherwise skip updating and result in
+						 * success anyway.
+						 */
+						if ($fields_to_update) {
+							$lld_macro_paths_to_update[] = $fields_to_update
+								+ ['lld_macro_pathid' => $lld_macro_path['lld_macro_pathid']];
+						}
+
+						/*
+						 * Remove processed LLD macros from the list. Macros left in $db_lld_macro_paths will be removed
+						 * afterwards.
+						 */
+						unset($db_lld_macro_paths[$idx2]);
+						unset($lld_macro_paths[$idx1]);
+					}
+					// Incorrect "lld_macro_pathid" cannot be given due to validation done previously.
+				}
+				else {
+					// Add or update fields by given "lld_macro".
+
+					if (bccomp($lld_macro_path['itemid'], $db_lld_macro_path['itemid']) == 0) {
+						if ($lld_macro_path['lld_macro'] === $db_lld_macro_path['lld_macro']) {
+							// If same "lld_macro" is given, add primary key and update only "path", if necessary.
+
+							if ($lld_macro_path['path'] !== $db_lld_macro_path['path']) {
+								$lld_macro_paths_to_update[] = [
+									'lld_macro_pathid' => $db_lld_macro_path['lld_macro_pathid'],
+									'path' => $lld_macro_path['path']
+								];
+							}
+
+							/*
+							 * Remove processed LLD macros from the list. Macros left in $db_lld_macro_paths will
+							 * be removed afterwards. And macros left in $lld_macro_paths will be created.
+							 */
+							unset($db_lld_macro_paths[$idx2]);
+							unset($lld_macro_paths[$idx1]);
+						}
+					}
+				}
+			}
+		}
+
+		// After all data has been collected, proceed with record update in DB.
+		$lld_macro_pathids_to_delete = zbx_objectValues($db_lld_macro_paths, 'lld_macro_pathid');
+
+		if ($lld_macro_pathids_to_delete) {
+			DB::delete('lld_macro_path', ['lld_macro_pathid' => $lld_macro_pathids_to_delete]);
+		}
+
+		if ($lld_macro_paths_to_update) {
+			$data = [];
+
+			foreach ($lld_macro_paths_to_update as $lld_macro_path) {
+				$data[] = [
+					'values' => $lld_macro_path,
+					'where' => [
+						'lld_macro_pathid' => $lld_macro_path['lld_macro_pathid']
+					]
+				];
+			}
+
+			DB::update('lld_macro_path', $data);
+		}
+
+		DB::insertBatch('lld_macro_path', $lld_macro_paths);
+
+		$this->updateItemPreprocessing($items);
 	}
 
 	/**
@@ -1010,12 +1220,6 @@ class CDiscoveryRule extends CItemGeneral {
 			$item['flags'] = ZBX_FLAG_DISCOVERY_RULE;
 			$item['value_type'] = ITEM_VALUE_TYPE_TEXT;
 
-			if (array_key_exists('type', $item) && $item['type'] == ITEM_TYPE_DEPENDENT) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value "%1$s" for "%2$s" field.',
-					ITEM_TYPE_DEPENDENT, 'type'
-				));
-			}
-
 			// unset fields that are updated using the 'filter' parameter
 			unset($item['evaltype']);
 			unset($item['formula']);
@@ -1134,9 +1338,196 @@ class CDiscoveryRule extends CItemGeneral {
 				_s('Incorrect value for field "%1$s": %2$s.', 'lifetime', $error)
 			);
 		}
+	}
 
-		if (array_key_exists('preprocessing', $item)) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Item pre-processing is not allowed for discovery rules.'));
+	/**
+	 * Checks if LLD macros contain duplicate names in "lld_macro".
+	 *
+	 * @param array  $lld_macro_paths                 Array of items to validate.
+	 * @param string $lld_macro_paths[]['lld_macro']  LLD macro string (optional for update method).
+	 * @param array  $macro_names                     Array where existing macro names are collected.
+	 * @param string $path                            Path to API object.
+	 *
+	 * @throws APIException if same discovery rules contains duplicate LLD macro names.
+	 */
+	protected function checkDuplicateLLDMacros(array $lld_macro_paths, $macro_names, $path) {
+		foreach ($lld_macro_paths as $num => $lld_macro_path) {
+			if (array_key_exists('lld_macro', $lld_macro_path)) {
+				if (array_key_exists($lld_macro_path['lld_macro'], $macro_names)) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Invalid parameter "%1$s": %2$s.', $path.'/lld_macro_paths/'.($num + 1).'/lld_macro',
+							_s('value "%1$s" already exists', $lld_macro_path['lld_macro'])
+						)
+					);
+				}
+
+				$macro_names[$lld_macro_path['lld_macro']] = true;
+			}
+		}
+	}
+
+	/**
+	 * Validates parameters in "lld_macro_paths" property for each item in create method.
+	 *
+	 * @param array  $items                                      Array of items to validate.
+	 * @param array  $items[]['lld_macro_paths']                 Array of LLD macro paths to validate for each
+	 *                                                           discovery rule (optional).
+	 * @param string $items[]['lld_macro_paths'][]['lld_macro']  LLD macro string. Required if "lld_macro_paths" exists.
+	 * @param string $items[]['lld_macro_paths'][]['path']       Path string. Validates as regular string. Required if
+	 *                                                           "lld_macro_paths" exists.
+	 *
+	 * @throws APIException if incorrect fields and values given.
+	 */
+	protected function validateCreateLLDMacroPaths(array $items) {
+		$rules = [
+			'lld_macro_paths' =>	['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY, 'fields' => [
+				'lld_macro' =>			['type' => API_LLD_MACRO, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('lld_macro_path', 'lld_macro')],
+				'path' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('lld_macro_path', 'path')]
+			]]
+		];
+
+		foreach ($items as $key => $item) {
+			if (array_key_exists('lld_macro_paths', $item)) {
+				$item = array_intersect_key($item, $rules);
+				$path = '/'.($key + 1);
+
+				if (!CApiInputValidator::validate(['type' => API_OBJECT, 'fields' => $rules], $item, $path, $error)) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+				}
+
+				$this->checkDuplicateLLDMacros($item['lld_macro_paths'], [], $path);
+			}
+		}
+	}
+
+	/**
+	 * Validates parameters in "lld_macro_paths" property for each item in create method.
+	 *
+	 * @param array  $items                                             Array of items to validate.
+	 * @param array  $items[]['lld_macro_paths']                        Array of LLD macro paths to validate for each
+	 *                                                                  discovery rule (optional).
+	 * @param string $items[]['lld_macro_paths'][]['lld_macro_pathid']  LLD macro path ID from DB (optional).
+	 * @param string $items[]['lld_macro_paths'][]['lld_macro']         LLD macro string. Required if "lld_macro_pathid"
+	 *                                                                  does not exist.
+	 * @param string $items[]['lld_macro_paths'][]['path']              Path string. Validates as regular string.
+	 *                                                                  Required if "lld_macro_pathid" and "lld_macro"
+	 *                                                                  do not exist.
+	 *
+	 * @throws APIException if incorrect fields and values given.
+	 */
+	protected function validateUpdateLLDMacroPaths(array $items) {
+		$rules = [
+			'lld_macro_paths' =>	['type' => API_OBJECTS, 'fields' => [
+				'lld_macro_pathid' =>	['type' => API_ID],
+				'lld_macro' =>			['type' => API_LLD_MACRO, 'length' => DB::getFieldLength('lld_macro_path', 'lld_macro')],
+				'path' =>				['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('lld_macro_path', 'path')]
+			]]
+		];
+
+		$items = $this->extendObjects('items', $items, ['templateid']);
+
+		foreach ($items as $key => $item) {
+			if (array_key_exists('lld_macro_paths', $item)) {
+				$itemid = $item['itemid'];
+				$templateid = $item['templateid'];
+
+				$item = array_intersect_key($item, $rules);
+				$path = '/'.($key + 1);
+
+				if (!CApiInputValidator::validate(['type' => API_OBJECT, 'fields' => $rules], $item, $path, $error)) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+				}
+
+				if (array_key_exists('lld_macro_paths', $item)) {
+					if ($templateid != 0) {
+						self::exception(ZBX_API_ERROR_PARAMETERS,
+							_s('Invalid parameter "%1$s": %2$s.', $path.'/lld_macro_paths',
+								_('cannot update property for templated discovery rule')
+							)
+						);
+					}
+
+					$lld_macro_pathids = [];
+
+					// Check that fields exists, are not empty, do not duplicate and collect IDs to compare with DB.
+					foreach ($item['lld_macro_paths'] as $num => $lld_macro_path) {
+						$subpath = $num + 1;
+
+						// API_NOT_EMPTY will not work, so we need at least one field to be present.
+						if (!array_key_exists('lld_macro', $lld_macro_path)
+								&& !array_key_exists('path', $lld_macro_path)
+								&& !array_key_exists('lld_macro_pathid', $lld_macro_path)) {
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_s('Invalid parameter "%1$s": %2$s.', $path.'/lld_macro_paths/'.$subpath,
+									_('cannot be empty')
+								)
+							);
+						}
+
+						// API 'uniq' => true will not work, because we validate API_ID not API_IDS. So make IDs unique.
+						if (array_key_exists('lld_macro_pathid', $lld_macro_path)) {
+							$lld_macro_pathids[$lld_macro_path['lld_macro_pathid']] = true;
+						}
+						else {
+							/*
+							 * In case "lld_macro_pathid" does not exist, we need to treat it as a new LLD macro with
+							 * both fields present.
+							 */
+							if (array_key_exists('lld_macro', $lld_macro_path)
+									&& !array_key_exists('path', $lld_macro_path)) {
+								self::exception(ZBX_API_ERROR_PARAMETERS,
+									_s('Invalid parameter "%1$s": %2$s.', $path.'/lld_macro_paths/'.$subpath,
+										_s('the parameter "%1$s" is missing', 'path')
+									)
+								);
+							}
+							elseif (array_key_exists('path', $lld_macro_path)
+									&& !array_key_exists('lld_macro', $lld_macro_path)) {
+								self::exception(ZBX_API_ERROR_PARAMETERS,
+									_s('Invalid parameter "%1$s": %2$s.', $path.'/lld_macro_paths/'.$subpath,
+										_s('the parameter "%1$s" is missing', 'lld_macro')
+									)
+								);
+							}
+						}
+					}
+
+					$this->checkDuplicateLLDMacros($item['lld_macro_paths'], [], $path);
+
+					/*
+					 * Validate "lld_macro_pathid" field. If "lld_macro_pathid" doesn't correspond to given "itemid"
+					 * or does not exist, throw an exception.
+					 */
+					if ($lld_macro_pathids) {
+						$lld_macro_pathids = array_keys($lld_macro_pathids);
+
+						$db_lld_macro_paths = DBfetchArrayAssoc(DBselect(
+							'SELECT lmp.lld_macro_pathid,lmp.lld_macro'.
+							' FROM lld_macro_path lmp'.
+							' WHERE lmp.itemid='.zbx_dbstr($itemid).
+								' AND '.dbConditionId('lmp.lld_macro_pathid', $lld_macro_pathids)
+						), 'lld_macro_pathid');
+
+						if (count($db_lld_macro_paths) != count($lld_macro_pathids)) {
+							self::exception(ZBX_API_ERROR_PERMISSIONS,
+								_('No permissions to referred object or it does not exist!')
+							);
+						}
+
+						$macro_names = [];
+
+						foreach ($item['lld_macro_paths'] as $num => $lld_macro_path) {
+							if (array_key_exists('lld_macro_pathid', $lld_macro_path)
+									&& !array_key_exists('lld_macro', $lld_macro_path)) {
+								$db_lld_macro_path = $db_lld_macro_paths[$lld_macro_path['lld_macro_pathid']];
+								$macro_names[$db_lld_macro_path['lld_macro']] = true;
+							}
+						}
+
+						$this->checkDuplicateLLDMacros($item['lld_macro_paths'], $macro_names, $path);
+					}
+				}
+			}
 		}
 	}
 
@@ -1163,9 +1554,11 @@ class CDiscoveryRule extends CItemGeneral {
 				'snmpv3_authprotocol', 'snmpv3_privprotocol', 'snmpv3_contextname', 'jmx_endpoint', 'url',
 				'query_fields', 'timeout', 'posts', 'status_codes', 'follow_redirects', 'post_type', 'http_proxy',
 				'headers', 'retrieve_mode', 'request_method', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password',
-				'verify_peer', 'verify_host', 'allow_traps'
+				'verify_peer', 'verify_host', 'allow_traps', 'master_itemid'
 			],
 			'selectFilter' => ['evaltype', 'formula', 'conditions'],
+			'selectLLDMacroPaths' => ['lld_macro', 'path'],
+			'selectPreprocessing' => ['type', 'params', 'error_handler', 'error_handler_params'],
 			'preservekeys' => true
 		]);
 		$srcDiscovery = reset($srcDiscovery);
@@ -1191,6 +1584,10 @@ class CDiscoveryRule extends CItemGeneral {
 			unset($condition);
 		}
 
+		if (!$dstDiscovery['lld_macro_paths']) {
+			unset($dstDiscovery['lld_macro_paths']);
+		}
+
 		// if this is a plain host, map discovery interfaces
 		if ($srcHost['status'] != HOST_STATUS_TEMPLATE) {
 			// find a matching interface
@@ -1208,6 +1605,25 @@ class CDiscoveryRule extends CItemGeneral {
 			}
 		}
 
+		// Master item should exists for LLD rule with type dependent item.
+		if ($srcDiscovery['type'] == ITEM_TYPE_DEPENDENT) {
+			$master_items = DBfetchArray(DBselect(
+				'SELECT i1.itemid'.
+				' FROM items i1,items i2'.
+				' WHERE i1.key_=i2.key_'.
+					' AND i1.hostid='.zbx_dbstr($dstDiscovery['hostid']).
+					' AND i2.itemid='.zbx_dbstr($srcDiscovery['master_itemid'])
+			));
+
+			if (!$master_items) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_s('Discovery rule "%1$s" cannot be copied without its master item.', $srcDiscovery['name'])
+				);
+			}
+
+			$dstDiscovery['master_itemid'] = $master_items[0]['itemid'];
+		}
+
 		// save new discovery
 		$newDiscovery = $this->create([$dstDiscovery]);
 		$dstDiscovery['itemid'] = $newDiscovery['itemids'][0];
@@ -1262,7 +1678,7 @@ class CDiscoveryRule extends CItemGeneral {
 			],
 			'selectApplications' => ['applicationid'],
 			'selectApplicationPrototypes' => ['name'],
-			'selectPreprocessing' => ['type', 'params'],
+			'selectPreprocessing' => ['type', 'params', 'error_handler', 'error_handler_params'],
 			'discoveryids' => $srcDiscovery['itemid'],
 			'preservekeys' => true
 		]);
@@ -1553,10 +1969,9 @@ class CDiscoveryRule extends CItemGeneral {
 	protected function copyHostPrototypes($srcid, array $dstDiscovery) {
 		$prototypes = API::HostPrototype()->get([
 			'discoveryids' => $srcid,
-			'output' => ['host', 'name', 'status'],
+			'output' => ['host', 'name', 'status', 'inventory_mode'],
 			'selectGroupLinks' => ['groupid'],
 			'selectGroupPrototypes' => ['name'],
-			'selectInventory' => ['inventory_mode'],
 			'selectTemplates' => ['templateid'],
 			'preservekeys' => true
 		]);
@@ -1590,7 +2005,26 @@ class CDiscoveryRule extends CItemGeneral {
 	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
 		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
 
+		if ($this->outputIsRequested('state', $options['output'])
+				|| $this->outputIsRequested('error', $options['output'])
+				|| (is_array($options['search']) && array_key_exists('error', $options['search']))
+				|| (is_array($options['filter']) && array_key_exists('state', $options['filter']))) {
+			$sqlParts['left_join']['item_rtdata'] = ['from' => 'item_rtdata ir', 'on' => 'ir.itemid=i.itemid'];
+			$sqlParts['left_table'] = $tableName;
+		}
+
 		if (!$options['countOutput']) {
+			if ($this->outputIsRequested('state', $options['output'])) {
+				$sqlParts = $this->addQuerySelect('ir.state', $sqlParts);
+			}
+			if ($this->outputIsRequested('error', $options['output'])) {
+				/*
+				 * SQL func COALESCE use for template items because they don't have record
+				 * in item_rtdata table and DBFetch convert null to '0'
+				 */
+				$sqlParts = $this->addQuerySelect(dbConditionCoalesce('ir.error', '', 'error'), $sqlParts);
+			}
+
 			// add filter fields
 			if ($this->outputIsRequested('formula', $options['selectFilter'])
 					|| $this->outputIsRequested('eval_formula', $options['selectFilter'])
@@ -1638,7 +2072,7 @@ class CDiscoveryRule extends CItemGeneral {
 
 				$items = zbx_toHash($items, 'parent_itemid');
 				foreach ($result as $itemid => $item) {
-					$result[$itemid]['items'] = isset($items[$itemid]) ? $items[$itemid]['rowscount'] : 0;
+					$result[$itemid]['items'] = array_key_exists($itemid, $items) ? $items[$itemid]['rowscount'] : '0';
 				}
 			}
 		}
@@ -1674,7 +2108,9 @@ class CDiscoveryRule extends CItemGeneral {
 
 				$triggers = zbx_toHash($triggers, 'parent_itemid');
 				foreach ($result as $itemid => $item) {
-					$result[$itemid]['triggers'] = isset($triggers[$itemid]) ? $triggers[$itemid]['rowscount'] : 0;
+					$result[$itemid]['triggers'] = array_key_exists($itemid, $triggers)
+						? $triggers[$itemid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1710,7 +2146,9 @@ class CDiscoveryRule extends CItemGeneral {
 
 				$graphs = zbx_toHash($graphs, 'parent_itemid');
 				foreach ($result as $itemid => $item) {
-					$result[$itemid]['graphs'] = isset($graphs[$itemid]) ? $graphs[$itemid]['rowscount'] : 0;
+					$result[$itemid]['graphs'] = array_key_exists($itemid, $graphs)
+						? $graphs[$itemid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1737,11 +2175,31 @@ class CDiscoveryRule extends CItemGeneral {
 				$hostPrototypes = zbx_toHash($hostPrototypes, 'parent_itemid');
 
 				foreach ($result as $itemid => $item) {
-					$result[$itemid]['hostPrototypes'] = isset($hostPrototypes[$itemid]) ? $hostPrototypes[$itemid]['rowscount'] : 0;
+					$result[$itemid]['hostPrototypes'] = array_key_exists($itemid, $hostPrototypes)
+						? $hostPrototypes[$itemid]['rowscount']
+						: '0';
 				}
 			}
 		}
 
+		if ($options['selectApplicationPrototypes'] !== null
+				&& $options['selectApplicationPrototypes'] != API_OUTPUT_COUNT) {
+			$relation_map = $this->createRelationMap($result, 'itemid', 'application_prototypeid',
+				'application_prototype'
+			);
+
+			$application_prototypes = API::getApiService()->select('application_prototype', [
+				'output' => $options['selectApplicationPrototypes'],
+				'filter' => ['application_prototypeid' => $relation_map->getRelatedIds()],
+				'limit' => $options['limitSelects'],
+				'preservekeys' => true
+			]);
+
+			$result = $relation_map->mapMany($result, $application_prototypes, 'applicationPrototypes',
+				$options['limitSelects']
+			);
+		}
+
 		if ($options['selectFilter'] !== null) {
 			$formulaRequested = $this->outputIsRequested('formula', $options['selectFilter']);
 			$evalFormulaRequested = $this->outputIsRequested('eval_formula', $options['selectFilter']);
@@ -1811,6 +2269,30 @@ class CDiscoveryRule extends CItemGeneral {
 			unset($rule);
 		}
 
+		// Add LLD macro paths.
+		if ($options['selectLLDMacroPaths'] !== null && $options['selectLLDMacroPaths'] != API_OUTPUT_COUNT) {
+			$lld_macro_paths = API::getApiService()->select('lld_macro_path', [
+				'output' => $this->outputExtend($options['selectLLDMacroPaths'], ['itemid', 'lld_macro_pathid']),
+				'filter' => ['itemid' => $itemIds],
+			]);
+
+			foreach ($result as &$lld_macro_path) {
+				$lld_macro_path['lld_macro_paths'] = [];
+			}
+			unset($lld_macro_path);
+
+			foreach ($lld_macro_paths as $lld_macro_path) {
+				$itemid = $lld_macro_path['itemid'];
+
+				if (!$this->outputIsRequested('lld_macro_pathid', $options['selectLLDMacroPaths'])) {
+					unset($lld_macro_path['lld_macro_pathid']);
+				}
+				unset($lld_macro_path['itemid']);
+
+				$result[$itemid]['lld_macro_paths'][] = $lld_macro_path;
+			}
+		}
+
 		return $result;
 	}
 }
