@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -58,40 +58,24 @@ abstract class CMapElement extends CApiService {
 			}
 
 			if (array_key_exists('urls', $selement)) {
+				$url_validate_options = ['allow_user_macro' => false];
+				if ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_HOST) {
+					$url_validate_options['allow_inventory_macro'] = INVENTORY_URL_MACRO_HOST;
+				}
+				elseif ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_TRIGGER) {
+					$url_validate_options['allow_inventory_macro'] = INVENTORY_URL_MACRO_TRIGGER;
+				}
+				else {
+					$url_validate_options['allow_inventory_macro'] = INVENTORY_URL_MACRO_NONE;
+				}
+
 				foreach ($selement['urls'] as $url_data) {
-					if (!CHtmlUrlValidator::validate($url_data['url'], false)) {
+					if (!CHtmlUrlValidator::validate($url_data['url'], $url_validate_options)) {
 						self::exception(ZBX_API_ERROR_PARAMETERS, _('Wrong value for url field.'));
 					}
 				}
 			}
 
-			/*
-			 * @deprecated  As of version 3.4, use elements data array.
-			 */
-			if (array_key_exists('elementid', $selement)) {
-				$this->deprecated('elementid is deprecated parameter.');
-
-				switch ($selement['elementtype']) {
-					case SYSMAP_ELEMENT_TYPE_HOST_GROUP:
-						$selement['elements'][0]['groupid'] = $selement['elementid'];
-						break;
-
-					case SYSMAP_ELEMENT_TYPE_HOST:
-						$selement['elements'][0]['hostid'] = $selement['elementid'];
-						break;
-
-					case SYSMAP_ELEMENT_TYPE_TRIGGER:
-						$selement['elements'][0]['triggerid'] = $selement['elementid'];
-						break;
-
-					case SYSMAP_ELEMENT_TYPE_MAP:
-						$selement['elements'][0]['sysmapid'] = $selement['elementid'];
-						break;
-				}
-
-				unset($selement['elementid']);
-			}
-
 			if ($update && array_key_exists('selementid', $selement)
 						&& array_key_exists($selement['selementid'], $db_selements)) {
 				$db_selement = $db_selements[$selement['selementid']];
@@ -124,12 +108,6 @@ abstract class CMapElement extends CApiService {
 				}
 			}
 
-			if (array_key_exists('elements', $selement)) {
-				if (!is_array($selement['elements'])) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
-				}
-			}
-
 			if ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_TRIGGER) {
 				foreach ($selement['elements'] as $element) {
 					if (!array_key_exists('triggerid', $element)) {
@@ -757,7 +735,7 @@ abstract class CMapElement extends CApiService {
 
 		foreach ($linkTriggers as $linkTrigger) {
 			if (!check_db_fields($linkTriggerDbFields, $linkTrigger)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('Wrong fields for linktrigger.'));
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
 			}
 
 			if (isset($linkTrigger['color']) && !$colorValidator->validate($linkTrigger['color'])) {
@@ -792,7 +770,7 @@ abstract class CMapElement extends CApiService {
 
 		foreach ($linkTriggers as $linkTrigger) {
 			if (!check_db_fields($linkTriggerDbFields, $linkTrigger)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('Wrong fields for linktrigger update.'));
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
 			}
 
 			if (isset($linkTrigger['color']) && !$colorValidator->validate($linkTrigger['color'])) {
@@ -817,7 +795,7 @@ abstract class CMapElement extends CApiService {
 
 		foreach ($linkTriggers as $linkTrigger) {
 			if (!check_db_fields($linktriggerDbFields, $linkTrigger)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('Wrong fields for linktrigger delete.'));
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
 			}
 		}
 	}
