@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ function DBconnect(&$error) {
 	$result = true;
 
 	$DB['DB'] = null; // global db handler
-	$DB['TRANSACTIONS'] = 0; // level of a nested transation
+	$DB['TRANSACTIONS'] = 0; // level of a nested transaction
 	$DB['TRANSACTION_NO_FAILED_SQLS'] = true; // true - if no statements failed in transaction, false - there are failed statements
 	$DB['SELECT_COUNT'] = 0; // stats
 	$DB['EXECUTE_COUNT'] = 0;
@@ -184,10 +184,6 @@ function DBconnect(&$error) {
 		$DB['DB'] = null;
 	}
 
-	if (!$result && !ZBX_SHOW_TECHNICAL_ERRORS) {
-		$error = _('System error occurred. Please contact Zabbix administrator.');
-	}
-
 	return $result;
 }
 
@@ -407,7 +403,11 @@ function DBselect($query, $limit = null, $offset = 0) {
 		$DB['TRANSACTION_NO_FAILED_SQLS'] = false;
 	}
 
-	CProfiler::getInstance()->profileSql(microtime(true) - $time_start, $query);
+	if (CApiService::$userData !== null && array_key_exists('debug_mode', CApiService::$userData)
+			&& CApiService::$userData['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
+		CProfiler::getInstance()->profileSql(microtime(true) - $time_start, $query);
+	}
+
 	return $result;
 }
 
@@ -427,7 +427,7 @@ function DBselect($query, $limit = null, $offset = 0) {
  * SELECT a FROM tbl LIMIT 10 OFFSET 5
  *
  * Oracle, DB2:
- * SELECT a FROM tbe WHERE rownum < 15 // ONLY < 15
+ * SELECT a FROM tbl WHERE rownum < 15 // ONLY < 15
  * SELECT * FROM (SELECT * FROM tbl) WHERE rownum BETWEEN 6 AND 15
  *
  * @param $query
@@ -519,7 +519,11 @@ function DBexecute($query, $skip_error_messages = 0) {
 		$DB['TRANSACTION_NO_FAILED_SQLS'] = false;
 	}
 
-	CProfiler::getInstance()->profileSql(microtime(true) - $time_start, $query);
+	if (CApiService::$userData !== null && array_key_exists('debug_mode', CApiService::$userData)
+			&& CApiService::$userData['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
+		CProfiler::getInstance()->profileSql(microtime(true) - $time_start, $query);
+	}
+
 	return (bool) $result;
 }
 
@@ -527,7 +531,7 @@ function DBexecute($query, $skip_error_messages = 0) {
  * Returns the next data set from a DB resource or false if there are no more results.
  *
  * @param resource $cursor
- * @param bool $convertNulls	convert all null values to string zeroes
+ * @param bool     $convertNulls  Convert all null values to string zeros.
  *
  * @return array|bool
  */
@@ -773,106 +777,125 @@ function check_db_fields($dbFields, &$args) {
  * In some frontend places we can get array with bool as input values parameter. This is fail!
  * Therefore we need check it and return 1=0 as temporary solution to not break the frontend.
  *
- * @param string $fieldName		field name to be used in SQL WHERE condition
- * @param array  $values		array of numerical values sorted in ascending order to be included in WHERE
- * @param bool   $notIn			builds inverted condition
- * @param bool   $sort			values mandatory must be sorted
- * @param bool   $quote
+ * @param string $field_name    Field name to be used in SQL WHERE condition
+ * @param array  $values        Array of numerical values sorted in ascending order to be included in WHERE
+ * @param bool   $not_in        Builds inverted condition
+ * @param bool   $zero_to_null  Cast zero to null
  *
  * @return string
  */
-function dbConditionInt($fieldName, array $values, $notIn = false, $sort = true, $quote = true) {
-	$MAX_EXPRESSIONS = 950; // maximum  number of values for using "IN (id1>,<id2>,...,<idN>)"
-	$MIN_NUM_BETWEEN = 4; // minimum number of consecutive values for using "BETWEEN <id1> AND <idN>"
+function dbConditionInt($field_name, array $values, $not_in = false, $zero_to_null = false) {
+	global $DB;
+
+	$MIN_NUM_BETWEEN = 4; // Minimum number of consecutive values for using "BETWEEN <id1> AND <idN>".
+	$MAX_NUM_IN = 950; // Maximum number of values for using "IN (<id1>,<id2>,...,<idN>)".
 
 	if (is_bool(reset($values))) {
 		return '1=0';
 	}
 
-	$values = array_keys(array_flip($values));
+	$values = array_flip($values);
 
-	if ($sort) {
-		natsort($values);
+	$has_zero = false;
 
-		$values = array_values($values);
+	if ($zero_to_null && array_key_exists(0, $values)) {
+		$has_zero = true;
+		unset($values[0]);
 	}
 
-	$betweens = [];
-	$data = [];
+	$values = array_keys($values);
+	natsort($values);
+	$values = array_values($values);
 
-	for ($i = 0, $size = count($values); $i < $size; $i++) {
-		$between = [];
+	$intervals = [];
+	$singles = [];
 
-		// analyze by chunk
-		if (isset($values[$i + $MIN_NUM_BETWEEN])
-				&& bccomp(bcadd($values[$i], $MIN_NUM_BETWEEN), $values[$i + $MIN_NUM_BETWEEN]) == 0) {
-			for ($sizeMinBetween = $i + $MIN_NUM_BETWEEN; $i < $sizeMinBetween; $i++) {
-				$between[] = $values[$i];
+	if ($DB['TYPE'] == ZBX_DB_ORACLE) {
+		// For better performance, use "BETWEEN" constructs for sequential integer values, for Oracle database.
+
+		for ($i = 0, $size = count($values); $i < $size; $i++) {
+			if ($i + $MIN_NUM_BETWEEN < $size && bcsub($values[$i + $MIN_NUM_BETWEEN], $values[$i]) == $MIN_NUM_BETWEEN) {
+				$interval_first = $values[$i];
+
+				// Search for the last sequential integer value.
+				for ($i += $MIN_NUM_BETWEEN; $i < $size && bcsub($values[$i], $values[$i - 1]) == 1; $i++);
+				$i--;
+
+				$interval_last = $values[$i];
+
+				// Save the first and last values of the sequential interval.
+				$intervals[] = [dbQuoteInt($interval_first), dbQuoteInt($interval_last)];
 			}
-
-			$i--; // shift 1 back
-
-			// analyze by one
-			for (; $i < $size; $i++) {
-				if (isset($values[$i + 1]) && bccomp(bcadd($values[$i], 1), $values[$i + 1]) == 0) {
-					$between[] = $values[$i + 1];
-				}
-				else {
-					break;
-				}
+			else {
+				$singles[] = dbQuoteInt($values[$i]);
 			}
-
-			$betweens[] = $between;
 		}
-		else {
-			$data[] = $values[$i];
-		}
-	}
-
-	// concatenate conditions
-	$dataSize = count($data);
-	$betweenSize = count($betweens);
-
-	$condition = '';
-	$operatorAnd = $notIn ? ' AND ' : ' OR ';
-
-	if ($betweens) {
-		$operatorNot = $notIn ? 'NOT ' : '';
-
-		foreach ($betweens as $between) {
-			$between = $quote
-				? $operatorNot.$fieldName.' BETWEEN '.zbx_dbstr($between[0]).' AND '.zbx_dbstr(end($between))
-				: $operatorNot.$fieldName.' BETWEEN '.$between[0].' AND '.end($between);
-
-			$condition .= $condition ? $operatorAnd.$between : $between;
-		}
-	}
-
-	if ($dataSize == 1) {
-		$operator = $notIn ? '!=' : '=';
-
-		$condition .= $quote
-			? ($condition ? $operatorAnd : '').$fieldName.$operator.zbx_dbstr($data[0])
-			: ($condition ? $operatorAnd : '').$fieldName.$operator.$data[0];
 	}
 	else {
-		$operatorNot = $notIn ? ' NOT' : '';
-		$data = array_chunk($data, $MAX_EXPRESSIONS);
+		// For better performance, use only "IN" constructs all other databases, except Oracle.
 
-		foreach ($data as $chunk) {
-			$chunkIns = '';
+		$singles = array_map(function($value) {
+			return dbQuoteInt($value);
+		}, $values);
+	}
 
-			foreach ($chunk as $value) {
-				$chunkIns .= $quote ? ','.zbx_dbstr($value) : ','.$value;
-			}
+	$condition = '';
 
-			$chunkIns = $fieldName.$operatorNot.' IN ('.substr($chunkIns, 1).')';
+	// Process intervals.
 
-			$condition .= $condition ? $operatorAnd.$chunkIns : $chunkIns;
+	foreach ($intervals as $interval) {
+		if ($condition !== '') {
+			$condition .= $not_in ? ' AND ' : ' OR ';
+		}
+
+		$condition .= ($not_in ? 'NOT ' : '').$field_name.' BETWEEN '.$interval[0].' AND '.$interval[1];
+	}
+
+	// Process individual values.
+
+	$single_chunks = array_chunk($singles, $MAX_NUM_IN);
+
+	foreach ($single_chunks as $chunk) {
+		if ($condition !== '') {
+			$condition .= $not_in ? ' AND ' : ' OR ';
+		}
+
+		if (count($chunk) == 1) {
+			$condition .= $field_name.($not_in ? '!=' : '=').$chunk[0];
+		}
+		else {
+			$condition .= $field_name.($not_in ? ' NOT' : '').' IN ('.implode(',', $chunk).')';
 		}
 	}
 
-	return (($dataSize && $betweenSize) || $betweenSize > 1 || $dataSize > $MAX_EXPRESSIONS) ? '('.$condition.')' : $condition;
+	if ($has_zero) {
+		if ($condition !== '') {
+			$condition .= $not_in ? ' AND ' : ' OR ';
+		}
+
+		$condition .= $field_name.($not_in ? ' IS NOT NULL' : ' IS NULL');
+	}
+
+	if (!$not_in) {
+		if ((int) $has_zero + count($intervals) + count($single_chunks) > 1) {
+			$condition = '('.$condition.')';
+		}
+	}
+
+	return $condition;
+}
+
+/**
+ * Takes an initial part of SQL query and appends a generated WHERE condition.
+ *
+ * @param string $fieldName		field name to be used in SQL WHERE condition
+ * @param array  $values		array of numerical values sorted in ascending order to be included in WHERE
+ * @param bool   $notIn			builds inverted condition
+ *
+ * @return string
+ */
+function dbConditionId($fieldName, array $values, $notIn = false) {
+	return dbConditionInt($fieldName, $values, $notIn, true);
 }
 
 /**
@@ -907,6 +930,44 @@ function dbConditionString($fieldName, array $values, $notIn = false) {
 	return '('.$fieldName.$in.'('.$condition.'))';
 }
 
+/**
+ * Quote a value if not an integer or out of BC Math bounds.
+ *
+ * @param mixed $value  Either the original or quoted value.
+ */
+function dbQuoteInt($value) {
+	if (!ctype_digit((string) $value) || bccomp($value, ZBX_MAX_UINT64) > 0) {
+		$value = zbx_dbstr($value);
+	}
+
+	return $value;
+}
+
+/**
+ * Return SQL for COALESCE like select. For fields with type NCHAR, NVARCHAR or NTEXT in Oracle NVL should be used
+ * instead of COALESCE because it will not check that all arguments have same type.
+ *
+ * @param string     $field_name       Field name to be used in returned query part.
+ * @param int|string $default_value    Default value to be returned.
+ * @param string     $alias            Alias to be used in 'AS' query part.
+ * @return string
+ */
+function dbConditionCoalesce($field_name, $default_value, $alias = '') {
+	global $DB;
+
+	if (is_string($default_value)) {
+		$default_value = ($default_value == '') ? '\'\'' : zbx_dbstr($default_value);
+	}
+
+	$query = (($DB['TYPE'] == ZBX_DB_ORACLE) ? 'NVL(' : 'COALESCE(').$field_name.','.$default_value.')';
+
+	if ($alias) {
+		$query .= ' AS '.$alias;
+	}
+
+	return $query;
+}
+
 /**
  * Transform DB cursor to array.
  *
@@ -938,20 +999,14 @@ function DBfetchArrayAssoc($cursor, $field) {
  *
  * @param resource $cursor
  * @param string   $column
- * @param bool     $asHash
  *
  * @return array
  */
-function DBfetchColumn($cursor, $column, $asHash = false) {
+function DBfetchColumn($cursor, $column) {
 	$result = [];
 
 	while ($dbResult = DBfetch($cursor)) {
-		if ($asHash) {
-			$result[$dbResult[$column]] = $dbResult[$column];
-		}
-		else {
-			$result[] = $dbResult[$column];
-		}
+		$result[] = $dbResult[$column];
 	}
 
 	return $result;
