@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,24 +20,27 @@
 
 
 require_once dirname(__FILE__).'/js/configuration.host.edit.js.php';
+require_once dirname(__FILE__).'/js/common.template.edit.js.php';
 
 $widget = (new CWidget())
 	->setTitle(_('Hosts'))
 	->addItem(get_header_host_table('', $data['hostid']));
 
 $divTabs = new CTabView();
+
 if (!hasRequest('form_refresh')) {
 	$divTabs->setSelected(0);
 }
 
 $frmHost = (new CForm())
+	->setId('hostsForm')
 	->setName('hostsForm')
+	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
 	->addVar('form', $data['form'])
 	->addVar('clear_templates', $data['clear_templates'])
 	->addVar('flags', $data['flags'])
-	->addVar('tls_connect', $data['tls_connect'])
-	->addVar('tls_accept', $data['tls_accept'])
-	->setAttribute('id', 'hostForm');
+	->addItem((new CVar('tls_connect', $data['tls_connect']))->removeId())
+	->addVar('tls_accept', $data['tls_accept']);
 
 if ($data['hostid'] != 0) {
 	$frmHost->addVar('hostid', $data['hostid']);
@@ -45,154 +48,57 @@ if ($data['hostid'] != 0) {
 if ($data['clone_hostid'] != 0) {
 	$frmHost->addVar('clone_hostid', $data['clone_hostid']);
 }
-if ($data['groupid'] != 0) {
-	$frmHost->addVar('groupid', $data['groupid']);
-}
 
 $hostList = new CFormList('hostlist');
 
 // LLD rule link
-if ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED) {
-	$hostList->addRow(_('Discovered by'),
-		new CLink($data['discoveryRule']['name'],
-			'host_prototypes.php?parent_discoveryid='.$data['discoveryRule']['itemid']
+if ($data['readonly']) {
+	$hostList->addRow(_('Discovered by'), $data['discoveryRule']
+		? new CLink($data['discoveryRule']['name'],
+			(new CUrl('host_prototypes.php'))
+				->setArgument('form', 'update')
+				->setArgument('parent_discoveryid', $data['discoveryRule']['itemid'])
+				->setArgument('hostid', $data['hostDiscovery']['parent_hostid'])
 		)
+		: (new CSpan(_('Inaccessible discovery rule')))->addClass(ZBX_STYLE_GREY)
 	);
 }
 
-$hostList->addRow(_('Host name'),
-	(new CTextBox('host', $data['host'], ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED), 128))
-		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-		->setAttribute('autofocus', 'autofocus')
-);
-
-$hostList->addRow(_('Visible name'),
-	(new CTextBox('visiblename', $data['visiblename'], ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED), 128))
-		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-);
-
-if ($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED) {
-	// groups for normal hosts
-	$groupsTB = new CTweenBox($frmHost, 'groups', $data['groups'], 10);
-
-	foreach ($data['groupsAll'] as $group) {
-		if (in_array($group['groupid'], $data['groups'])) {
-			$groupsTB->addItem($group['groupid'], $group['name'], null,
-				array_key_exists($group['groupid'], $data['groupsAllowed'])
-			);
-		}
-		elseif (array_key_exists($group['groupid'], $data['groupsAllowed'])) {
-			$groupsTB->addItem($group['groupid'], $group['name']);
-		}
-	}
-
-	$hostList->addRow(_('Groups'), $groupsTB->get(_('In groups'), _('Other groups')));
-
-	$new_group = (new CTextBox('newgroup', $data['newgroup']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
-	$new_group_label = _('New group');
-	if (CWebUser::$data['type'] != USER_TYPE_SUPER_ADMIN) {
-		$new_group_label .= ' '._('(Only super admins can create groups)');
-		$new_group->setReadonly(true);
-	}
-	$hostList->addRow(new CLabel($new_group_label, 'newgroup'),
-		(new CSpan($new_group))->addClass(ZBX_STYLE_FORM_NEW_GROUP)
-	);
-}
-else {
-	// groups for discovered hosts
-	$groupBox = new CListBox(null, null, 10);
-	$groupBox->setEnabled(false);
-	foreach ($data['groupsAll'] as $group) {
-		if (in_array($group['groupid'], $data['groups'])) {
-			$groupBox->addItem($group['groupid'], $group['name'], null,
-				array_key_exists($group['groupid'], $data['groupsAllowed'])
-			);
-		}
-	}
-	$hostList->addRow(_('Groups'), $groupBox);
-	$hostList->addVar('groups', $data['groups']);
-}
-
-// interfaces for normal hosts
-if ($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED) {
-	zbx_add_post_js($data['interfaces']
-		? 'hostInterfacesManager.add('.CJs::encodeJson($data['interfaces']).');'
-		: 'hostInterfacesManager.addNew("agent");');
-
-	// Zabbix agent interfaces
-	$ifTab = (new CTable())
-		->setId('agentInterfaces')
-		->setHeader([
-			new CColHeader(),
-			new CColHeader(_('IP address')),
-			new CColHeader(_('DNS name')),
-			new CColHeader(_('Connect to')),
-			new CColHeader(_('Port')),
-			(new CColHeader(_('Default')))->setColSpan(2)
-		])
-		->addRow((new CRow([
-			(new CCol(
-				(new CButton('addAgentInterface', _('Add')))->addClass(ZBX_STYLE_BTN_LINK)
-			))->setColSpan(7)
-		]))->setId('agentInterfacesFooter'));
-
-	$hostList->addRow(_('Agent interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'agent')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
+$hostList
+	->addRow(
+		(new CLabel(_('Host name'), 'host'))->setAsteriskMark(),
+		(new CTextBox('host', $data['host'], $data['readonly'], 128))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired()
+			->setAttribute('autofocus', 'autofocus')
+	)
+	->addRow(_('Visible name'),
+		(new CTextBox('visiblename', $data['visiblename'], $data['readonly'], 128))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+	)
+	->addRow((new CLabel(_('Groups'), 'groups__ms'))->setAsteriskMark(),
+		(new CMultiSelect([
+			'name' => 'groups[]',
+			'object_name' => 'hostGroup',
+			'disabled' => $data['readonly'],
+			'add_new' => (CWebUser::$data['type'] == USER_TYPE_SUPER_ADMIN),
+			'data' => $data['groups_ms'],
+			'popup' => [
+				'parameters' => [
+					'srctbl' => 'host_groups',
+					'srcfld1' => 'groupid',
+					'dstfrm' => $frmHost->getName(),
+					'dstfld1' => 'groups_',
+					'editable' => true
+				]
+			]
+		]))
+			->setAriaRequired()
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 	);
 
-	// SNMP interfaces
-	$ifTab = (new CTable())
-		->setId('SNMPInterfaces')
-		->addRow((new CRow([
-			(new CCol(
-				(new CButton('addSNMPInterface', _('Add')))->addClass(ZBX_STYLE_BTN_LINK)
-			))->setColSpan(7)
-		]))->setId('SNMPInterfacesFooter'));
-
-	$hostList->addRow(_('SNMP interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'snmp')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
-	);
-
-	// JMX interfaces
-	$ifTab = (new CTable())
-		->setId('JMXInterfaces')
-		->addRow((new CRow([
-			(new CCol(
-				(new CButton('addJMXInterface', _('Add')))->addClass(ZBX_STYLE_BTN_LINK)
-			))->setColSpan(7)
-		]))->setId('JMXInterfacesFooter'));
-
-	$hostList->addRow(_('JMX interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'jmx')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
-	);
-
-	// IPMI interfaces
-	$ifTab = (new CTable())
-		->setId('IPMIInterfaces')
-		->addRow((new CRow([
-			(new CCol(
-				(new CButton('addIPMIInterface', _('Add')))->addClass(ZBX_STYLE_BTN_LINK)
-			))->setColSpan(7)
-		]))->setId('IPMIInterfacesFooter'));
-
-	$hostList->addRow(_('IPMI interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'ipmi')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
-	);
-}
-// interfaces for discovered hosts
-else {
+// Interfaces for discovered hosts.
+if ($data['readonly']) {
 	$existingInterfaceTypes = [];
 	foreach ($data['interfaces'] as $interface) {
 		$existingInterfaceTypes[$interface['type']] = true;
@@ -202,7 +108,7 @@ else {
 
 	$hostList->addVar('interfaces', $data['interfaces']);
 
-	// Zabbix agent interfaces
+	// Zabbix agent interfaces.
 	$ifTab = (new CTable())
 		->setId('agentInterfaces')
 		->setHeader([
@@ -228,7 +134,7 @@ else {
 			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
 	);
 
-	// SNMP interfaces
+	// SNMP interfaces.
 	$ifTab = (new CTable())->setId('SNMPInterfaces');
 
 	$row = (new CRow())->setId('SNMPInterfacesFooter');
@@ -245,7 +151,7 @@ else {
 			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
 	);
 
-	// JMX interfaces
+	// JMX interfaces.
 	$ifTab = (new CTable())->setId('JMXInterfaces');
 
 	$row = (new CRow())->setId('JMXInterfacesFooter');
@@ -262,7 +168,7 @@ else {
 			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
 	);
 
-	// IPMI interfaces
+	// IPMI interfaces.
 	$ifTab = (new CTable())->setId('IPMIInterfaces');
 
 	$row = (new CRow())->setId('IPMIInterfacesFooter');
@@ -279,21 +185,106 @@ else {
 			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
 	);
 }
+// Interfaces for normal hosts.
+else {
+	zbx_add_post_js($data['interfaces']
+		? 'hostInterfacesManager.add('.CJs::encodeJson($data['interfaces']).');'
+		: 'hostInterfacesManager.addNew("agent");');
+
+	$hostList->addRow('',
+		(new CLabel(_('At least one interface must exist.')))->setAsteriskMark()
+	);
+	// Zabbix agent interfaces.
+	$ifTab = (new CTable())
+		->setId('agentInterfaces')
+		->setHeader([
+			new CColHeader(),
+			new CColHeader(_('IP address')),
+			new CColHeader(_('DNS name')),
+			new CColHeader(_('Connect to')),
+			new CColHeader(_('Port')),
+			(new CColHeader(_('Default')))->setColSpan(2)
+		])
+		->addRow((new CRow([
+			(new CCol(
+				(new CButton('addAgentInterface', _('Add')))->addClass(ZBX_STYLE_BTN_LINK)
+			))->setColSpan(7)
+		]))->setId('agentInterfacesFooter'));
+
+	$hostList->addRow(_('Agent interfaces'),
+		(new CDiv($ifTab))
+			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+			->setAttribute('data-type', 'agent')
+			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
+	);
+
+	// SNMP interfaces.
+	$ifTab = (new CTable())
+		->setId('SNMPInterfaces')
+		->addRow((new CRow([
+			(new CCol(
+				(new CButton('addSNMPInterface', _('Add')))->addClass(ZBX_STYLE_BTN_LINK)
+			))->setColSpan(7)
+		]))->setId('SNMPInterfacesFooter'));
+
+	$hostList->addRow(_('SNMP interfaces'),
+		(new CDiv($ifTab))
+			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+			->setAttribute('data-type', 'snmp')
+			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
+	);
+
+	// JMX interfaces.
+	$ifTab = (new CTable())
+		->setId('JMXInterfaces')
+		->addRow((new CRow([
+			(new CCol(
+				(new CButton('addJMXInterface', _('Add')))->addClass(ZBX_STYLE_BTN_LINK)
+			))->setColSpan(7)
+		]))->setId('JMXInterfacesFooter'));
+
+	$hostList->addRow(_('JMX interfaces'),
+		(new CDiv($ifTab))
+			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+			->setAttribute('data-type', 'jmx')
+			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
+	);
+
+	// IPMI interfaces.
+	$ifTab = (new CTable())
+		->setId('IPMIInterfaces')
+		->addRow((new CRow([
+			(new CCol(
+				(new CButton('addIPMIInterface', _('Add')))->addClass(ZBX_STYLE_BTN_LINK)
+			))->setColSpan(7)
+		]))->setId('IPMIInterfacesFooter'));
+
+	$hostList->addRow(_('IPMI interfaces'),
+		(new CDiv($ifTab))
+			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+			->setAttribute('data-type', 'ipmi')
+			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
+	);
+}
 
 $hostList->addRow(_('Description'),
-	(new CTextArea('description', $data['description']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+	(new CTextArea('description', $data['description']))
+		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		->setMaxlength(DB::getFieldLength('hosts', 'description'))
 );
 
 // Proxy
-if ($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED) {
-	$proxy = new CComboBox('proxy_hostid', $data['proxy_hostid'], null, [0 => _('(no proxy)')] + $data['proxies']);
-	$proxy->setEnabled($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED);
-}
-else {
-	$proxy = (new CTextBox(null, $data['proxy_hostid'] != 0 ? $data['proxies'][$data['proxy_hostid']] : _('(no proxy)'), true))
-		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
+if ($data['readonly']) {
+	$proxy = (new CTextBox(null,
+		($data['proxy_hostid'] == 0) ? _('(no proxy)') : $data['proxies'][$data['proxy_hostid']], true)
+	)->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
 	$hostList->addVar('proxy_hostid', $data['proxy_hostid']);
 }
+else {
+	$proxy = new CComboBox('proxy_hostid', $data['proxy_hostid'], null, [0 => _('(no proxy)')] + $data['proxies']);
+	$proxy->setEnabled(true);
+}
+
 $hostList->addRow(_('Monitored by proxy'), $proxy);
 
 $hostList->addRow(_('Enabled'),
@@ -556,16 +547,45 @@ $divTabs->addTab('hostTab', _('Host'), $hostList);
 // templates
 $tmplList = new CFormList();
 
-// templates for normal hosts
-if ($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED) {
-	$ignoredTemplates = [];
+// Templates for discovered hosts.
+if ($data['readonly']) {
+	$linkedTemplateTable = (new CTable())
+		->setHeader([_('Name')])
+		->addStyle('width: 100%;');
+
+	foreach ($data['linked_templates'] as $template) {
+		$tmplList->addVar('templates[]', $template['templateid']);
+
+		if (array_key_exists($template['templateid'], $data['writable_templates'])) {
+			$template_link = (new CLink($template['name'],
+				(new CUrl('templates.php'))
+					->setArgument('form','update')
+					->setArgument('templateid', $template['templateid'])
+			))->setTarget('_blank');
+		}
+		else {
+			$template_link = new CSpan($template['name']);
+		}
+
+		$linkedTemplateTable->addRow($template_link, null, 'conditions_'.$template['templateid']);
+	}
+
+	$tmplList->addRow(_('Linked templates'),
+		(new CDiv($linkedTemplateTable))
+			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+			->addStyle('min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
+	);
+}
+// Templates for normal hosts.
+else {
+	$disableids = [];
 
 	$linkedTemplateTable = (new CTable())
 		->setAttribute('style', 'width: 100%;')
 		->setHeader([_('Name'), _('Action')]);
 
 	foreach ($data['linked_templates'] as $template) {
-		$tmplList->addVar('templates[]', $template['templateid']);
+		$tmplList->addItem((new CVar('templates['.$template['templateid'].']', $template['templateid']))->removeId());
 
 		if (array_key_exists($template['templateid'], $data['writable_templates'])) {
 			$template_link = (new CLink($template['name'],
@@ -596,59 +616,38 @@ if ($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED) {
 			))->addClass(ZBX_STYLE_NOWRAP)
 		], null, 'conditions_'.$template['templateid']);
 
-		$ignoredTemplates[$template['templateid']] = $template['name'];
+		$disableids[] = $template['templateid'];
 	}
 
-	$tmplList->addRow(_('Linked templates'),
-		(new CDiv($linkedTemplateTable))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
-	);
+	$add_templates_ms = (new CMultiSelect([
+		'name' => 'add_templates[]',
+		'object_name' => 'templates',
+		'data' => $data['add_templates'],
+		'popup' => [
+			'parameters' => [
+				'srctbl' => 'templates',
+				'srcfld1' => 'hostid',
+				'srcfld2' => 'host',
+				'dstfrm' => $frmHost->getName(),
+				'dstfld1' => 'add_templates_',
+				'disableids' => $disableids
+			]
+		]
+	]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
 
-	// create new linked template table
-	$newTemplateTable = (new CTable())
-		->addRow([
-			(new CMultiSelect([
-				'name' => 'add_templates[]',
-				'objectName' => 'templates',
-				'ignored' => $ignoredTemplates,
-				'popup' => [
-					'parameters' => 'srctbl=templates&srcfld1=hostid&srcfld2=host&dstfrm='.$frmHost->getName().
-						'&dstfld1=add_templates_&templated_hosts=1&multiselect=1'
-				]
-			]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-		])
-		->addRow([
-			(new CSimpleButton(_('Add')))
-				->onClick('javascript: submitFormWithParam("'.$frmHost->getName().'", "add_template", "1");')
-				->addClass(ZBX_STYLE_BTN_LINK)
-		]);
-
-	$tmplList->addRow(_('Link new templates'),
-		(new CDiv($newTemplateTable))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
-	);
-}
-// templates for discovered hosts
-else {
-	$linkedTemplateTable = (new CTable())
-		->setAttribute('style', 'width: 100%;')
-		->setHeader([_('Name')]);
-
-	foreach ($data['linked_templates'] as $template) {
-		$tmplList->addVar('templates[]', $template['templateid']);
-		$template_link = (new CLink($template['name'], 'templates.php?form=update&templateid='.$template['templateid']))
-			->setTarget('_blank');
-
-		$linkedTemplateTable->addRow($template_link, null, 'conditions_'.$template['templateid']);
-	}
-
-	$tmplList->addRow(_('Linked templates'),
-		(new CDiv($linkedTemplateTable))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
-	);
+	$tmplList
+		->addRow(_('Linked templates'),
+			(new CDiv($linkedTemplateTable))
+				->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+				->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
+		)
+		->addRow((new CLabel(_('Link new templates'), 'add_templates__ms')),
+			(new CDiv(
+				(new CTable())->addRow([$add_templates_ms])
+			))
+				->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+				->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
+		);
 }
 
 $divTabs->addTab('templateTab', _('Templates'), $tmplList);
@@ -656,11 +655,7 @@ $divTabs->addTab('templateTab', _('Templates'), $tmplList);
 /*
  * IPMI
  */
-if ($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED) {
-	$cmbIPMIAuthtype = new CListBox('ipmi_authtype', $data['ipmi_authtype'], 7, null, ipmiAuthTypes());
-	$cmbIPMIPrivilege = new CListBox('ipmi_privilege', $data['ipmi_privilege'], 5, null, ipmiPrivileges());
-}
-else {
+if ($data['readonly']) {
 	$cmbIPMIAuthtype = [
 		(new CTextBox('ipmi_authtype_name', ipmiAuthTypes($data['ipmi_authtype']), true))
 			->setWidth(ZBX_TEXTAREA_SMALL_WIDTH),
@@ -672,31 +667,53 @@ else {
 		new CVar('ipmi_privilege', $data['ipmi_privilege'])
 	];
 }
+else {
+	$cmbIPMIAuthtype = new CListBox('ipmi_authtype', $data['ipmi_authtype'], 7, null, ipmiAuthTypes());
+	$cmbIPMIPrivilege = new CListBox('ipmi_privilege', $data['ipmi_privilege'], 5, null, ipmiPrivileges());
+}
 
 $divTabs->addTab('ipmiTab', _('IPMI'),
 	(new CFormList())
 		->addRow(_('Authentication algorithm'), $cmbIPMIAuthtype)
 		->addRow(_('Privilege level'), $cmbIPMIPrivilege)
 		->addRow(_('Username'),
-			(new CTextBox('ipmi_username', $data['ipmi_username'], ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED)))
+			(new CTextBox('ipmi_username', $data['ipmi_username'], $data['readonly']))
 				->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 		)
 		->addRow(_('Password'),
-			(new CTextBox('ipmi_password', $data['ipmi_password'], ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED)))
+			(new CTextBox('ipmi_password', $data['ipmi_password'], $data['readonly']))
 				->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 		)
 );
 
+/*
+ * Tags
+ */
+if (!$data['readonly']) {
+	$tags_view = new CView('configuration.tags.tab', [
+		'source' => 'host',
+		'tags' => $data['tags'],
+		'readonly' => false
+	]);
+	$divTabs->addTab('tags-tab', _('Tags'), $tags_view->render());
+}
+
 /*
  * Macros
  */
-$macrosView = new CView('hostmacros', [
-	'macros' => $data['macros'],
-	'show_inherited_macros' => $data['show_inherited_macros'],
-	'is_template' => false,
-	'readonly' => ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED)
-]);
-$divTabs->addTab('macroTab', _('Macros'), $macrosView->render());
+$divTabs->addTab('macroTab', _('Macros'),
+	(new CFormList('macrosFormList'))
+		->addRow(null, (new CRadioButtonList('show_inherited_macros', (int) $data['show_inherited_macros']))
+			->addValue(_('Host macros'), 0)
+			->addValue(_('Inherited and host macros'), 1)
+			->setModern(true)
+		)
+		->addRow(null, new CObject((new CView('hostmacros.list.html', [
+			'macros' => $data['macros'],
+			'show_inherited_macros' => $data['show_inherited_macros'],
+			'readonly' => $data['readonly']
+		]))->getOutput()), 'macros_container')
+);
 
 $inventoryFormList = new CFormList('inventorylist');
 
@@ -705,10 +722,10 @@ $inventoryFormList->addRow(null,
 		->addValue(_('Disabled'), HOST_INVENTORY_DISABLED)
 		->addValue(_('Manual'), HOST_INVENTORY_MANUAL)
 		->addValue(_('Automatic'), HOST_INVENTORY_AUTOMATIC)
-		->setEnabled($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED)
+		->setEnabled(!$data['readonly'])
 		->setModern(true)
 );
-if ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED) {
+if ($data['readonly']) {
 	$inventoryFormList->addVar('inventory_mode', $data['inventory_mode']);
 }
 
@@ -727,11 +744,9 @@ foreach ($hostInventoryFields as $inventoryNo => $inventoryInfo) {
 			->setWidth(ZBX_TEXTAREA_BIG_WIDTH);
 	}
 	else {
-		$field_length = $hostInventoryTable['fields'][$field_name]['length'];
-
 		$input = (new CTextBox('host_inventory['.$field_name.']', $data['host_inventory'][$field_name]))
-			->setWidth(($field_length < 39) ? ZBX_TEXTAREA_SMALL_WIDTH : ZBX_TEXTAREA_BIG_WIDTH)
-			->setAttribute('maxlength', $field_length);
+			->setWidth(ZBX_TEXTAREA_BIG_WIDTH)
+			->setAttribute('maxlength', $hostInventoryTable['fields'][$field_name]['length']);
 	}
 
 	if ($data['inventory_mode'] == HOST_INVENTORY_DISABLED) {
@@ -743,7 +758,7 @@ foreach ($hostInventoryFields as $inventoryNo => $inventoryInfo) {
 		$name = $data['inventory_items'][$inventoryNo]['name_expanded'];
 
 		$link = (new CLink($name, 'items.php?form=update&itemid='.$data['inventory_items'][$inventoryNo]['itemid']))
-			->setAttribute('title', _s('This field is automatically populated by item "%s".', $name));
+			->setTitle(_s('This field is automatically populated by item "%s".', $name));
 
 		$inventory_item = (new CSpan([' &larr; ', $link]))->addClass('populating_item');
 		if ($data['inventory_mode'] != HOST_INVENTORY_AUTOMATIC) {
@@ -764,7 +779,7 @@ foreach ($hostInventoryFields as $inventoryNo => $inventoryInfo) {
 	$inventoryFormList->addRow($inventoryInfo['title'], [$input, $inventory_item]);
 }
 
-$divTabs->addTab('inventoryTab', _('Host inventory'), $inventoryFormList);
+$divTabs->addTab('inventoryTab', _('Inventory'), $inventoryFormList);
 
 // Encryption form list.
 $encryption_form_list = (new CFormList('encryption'))
@@ -774,39 +789,43 @@ $encryption_form_list = (new CFormList('encryption'))
 			->addValue(_('PSK'), HOST_ENCRYPTION_PSK)
 			->addValue(_('Certificate'), HOST_ENCRYPTION_CERTIFICATE)
 			->setModern(true)
-			->setEnabled($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED)
+			->setEnabled(!$data['readonly'])
 	)
 	->addRow(_('Connections from host'),
 		(new CList())
 			->addClass(ZBX_STYLE_LIST_CHECK_RADIO)
 			->addItem((new CCheckBox('tls_in_none'))
 				->setLabel(_('No encryption'))
-				->setEnabled($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED)
+				->setEnabled(!$data['readonly'])
 			)
 			->addItem((new CCheckBox('tls_in_psk'))
 				->setLabel(_('PSK'))
-				->setEnabled($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED)
+				->setEnabled(!$data['readonly'])
 			)
 			->addItem((new CCheckBox('tls_in_cert'))
 				->setLabel(_('Certificate'))
-				->setEnabled($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED)
+				->setEnabled(!$data['readonly'])
 			)
 	)
-	->addRow(_('PSK identity'),
-		(new CTextBox('tls_psk_identity', $data['tls_psk_identity'], $data['flags'] == ZBX_FLAG_DISCOVERY_CREATED, 128))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+	->addRow(
+		(new CLabel(_('PSK identity'), 'tls_psk_identity'))->setAsteriskMark(),
+		(new CTextBox('tls_psk_identity', $data['tls_psk_identity'], $data['readonly'], 128))
+			->setWidth(ZBX_TEXTAREA_BIG_WIDTH)
+			->setAriaRequired()
 	)
-	->addRow(_('PSK'),
-		(new CTextBox('tls_psk', $data['tls_psk'], $data['flags'] == ZBX_FLAG_DISCOVERY_CREATED, 512))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+	->addRow(
+		(new CLabel(_('PSK'), 'tls_psk'))->setAsteriskMark(),
+		(new CTextBox('tls_psk', $data['tls_psk'], $data['readonly'], 512))
+			->setWidth(ZBX_TEXTAREA_BIG_WIDTH)
+			->setAriaRequired()
 	)
 	->addRow(_('Issuer'),
-		(new CTextBox('tls_issuer', $data['tls_issuer'], $data['flags'] == ZBX_FLAG_DISCOVERY_CREATED, 1024))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		(new CTextBox('tls_issuer', $data['tls_issuer'], $data['readonly'], 1024))
+			->setWidth(ZBX_TEXTAREA_BIG_WIDTH)
 	)
 	->addRow(_x('Subject', 'encryption certificate'),
-		(new CTextBox('tls_subject', $data['tls_subject'], $data['flags'] == ZBX_FLAG_DISCOVERY_CREATED, 1024))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		(new CTextBox('tls_subject', $data['tls_subject'], $data['readonly'], 1024))
+			->setWidth(ZBX_TEXTAREA_BIG_WIDTH)
 	);
 
 $divTabs->addTab('encryptionTab', _('Encryption'), $encryption_form_list);
@@ -834,7 +853,6 @@ else {
 }
 
 $frmHost->addItem($divTabs);
-
 $widget->addItem($frmHost);
 
 return $widget;
