@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -44,11 +44,11 @@ if ($data['triggerid'] !== null) {
 
 if ($data['limited']) {
 	$triggersForm
-		->addVar('opdata', $data['opdata'])
-		->addVar('recovery_mode', $data['recovery_mode'])
-		->addVar('type', $data['type'])
-		->addVar('correlation_mode', $data['correlation_mode'])
-		->addVar('manual_close', $data['manual_close']);
+		->addItem((new CVar('opdata', $data['opdata']))->removeId())
+		->addItem((new CVar('recovery_mode', $data['recovery_mode']))->removeId())
+		->addItem((new CVar('type', $data['type']))->removeId())
+		->addItem((new CVar('correlation_mode', $data['correlation_mode']))->removeId())
+		->addItem((new CVar('manual_close', $data['manual_close']))->removeId());
 }
 
 // create form list
@@ -75,11 +75,11 @@ $triggersFormList
 
 // append expression to form list
 if ($data['expression_field_readonly']) {
-	$triggersForm->addVar('expression', $data['expression']);
+	$triggersForm->addItem((new CVar('expression', $data['expression']))->removeId());
 }
 
 if ($data['recovery_expression_field_readonly']) {
-	$triggersForm->addVar('recovery_expression', $data['recovery_expression']);
+	$triggersForm->addItem((new CVar('recovery_expression', $data['recovery_expression']))->removeId());
 }
 
 $popup_options = [
